package edu.nju.controller;

import edu.nju.entities.Answer;
import edu.nju.entities.Item;
import edu.nju.entities.Job;
import edu.nju.service.ItemService;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

@Controller
@RequestMapping(value = "/item")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class ItemController {

    @Autowired
    ItemService iservice;

    @RequestMapping(value = "/getJob")
    @ResponseBody
    public void getJobById(String job_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            Job job = iservice.getJob(job_id);
            out.print(new JSONObject(job));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getItemsByJob")
    @ResponseBody
    public void getItemListByJobId(String job_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<Item> items = iservice.getJobItems(job_id);
            out.print(new JSONArray(items));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    @RequestMapping(value = "/getItemsByWorker")
    @ResponseBody
    public void getItemListByWorkerJob(String job_id, String worker_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<Item> items = iservice.getItemsByWorkerJob(worker_id, job_id);
            out.print(new JSONArray(items));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    @RequestMapping(value = "/getAnswerByWorkerItem")
    @ResponseBody
    public void getAnswerByWorkerItem(String item_id,String worker_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            Answer answer = iservice.getAnswerByItemWorker(item_id, worker_id);
            out.print(new JSONObject(answer));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getAnswersByWorkerJob")
    @ResponseBody
    public void getAnswersByWorkerJob(String job_id,String worker_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<Answer>answers=iservice.getAnswersByWorkerJob(job_id, worker_id);
            out.print(new JSONArray(answers));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/saveAnswer")
    @ResponseBody
    public void saveAnswer(String item_id, String worker_id,String job_id,List<String>answers,List<String> attachment_location, HttpServletResponse response){
        JSONObject result = new JSONObject();
        String id = iservice.saveAnswer(item_id, worker_id, job_id, answers, attachment_location);
        if(id.equals("")) {
            result.put("status", "200");
            result.put("id", id);
        }
        else {
            result.put("status", "500");
        }
        try {
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/updateAnswer")
    @ResponseBody
    public void updateAnswer(String id, String item_id, String worker_id, String job_id, List<String>answers,List<String> attachment_location, HttpServletResponse response){
        JSONObject result = new JSONObject();
        if(iservice.updateAnswer(id, item_id, worker_id, job_id, answers, attachment_location)){
            result.put("status", "200");
        } else {
            result.put("status", "500");
        }
        try {
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }



    

}
